AnalysisTask

Struct AnalysisTask 

pub struct AnalysisTask<'a, N: Grammar, H: TaskHandle = TriggerHandle, S: SyncBuildHasher = RandomState> { /* private fields */ }
A task that grants access to the semantic features of the Analyzer.

This kind of task implements a SemanticAccess trait through which you can read particular attribute values, but you cannot change the content of the documents.

You may have as many instances of this task as needed at the same time, and you can use them from multiple threads to read the attributes. The analyzer is capable to manage the semantic graph concurrently.