ExclusiveTask

Struct ExclusiveTask 

pub struct ExclusiveTask<'a, N: Grammar, H: TaskHandle = TriggerHandle, S: SyncBuildHasher = RandomState> { /* private fields */ }
An exclusive task that grants access to change documents and to observe the semantic graph of the Analyzer.

This task implements a MutationAccess trait through which you can create, delete, or edit the existing documents’ content, and implements a SemanticAccess trait through which you can read particular attribute values.

You can request both kinds of operations sequentially in a single thread, but the Analyzer does not allow you to have more than one active Exclusive task, and exclusive access is granted if and only if no other types of active tasks are granted.