pub struct TableReadGuard<'a, K, V, S = RandomState> { /* private fields */ }Expand description
A RAII guard, that provides read access to the Table entry’s value.
Created by the Table::get or Table::try_get methods.
The guard keeps the corresponding Table shard locked for read until the guard is dropped.