TableWriteGuard

Struct TableWriteGuard 

pub struct TableWriteGuard<'a, K, V, S = RandomState> { /* private fields */ }
A RAII guard, that provides read and write access to the Table entry’s value.

Created by the Table::get_mut method.

The guard keeps the corresponding Table shard locked for write until the guard is dropped.