VoidSyntax

Struct VoidSyntax 

pub struct VoidSyntax<T: Token> { /* private fields */ }
A Node without a syntax parser.

You are encouraged to use this object with the Document or the MutableUnit when you need an incremental lexical parser, but you don’t need a syntax parser: Document::<VoidSyntax<MyToken>>::new_mutable("foo bar baz").

This object makes the syntax parsing stage of the incremental reprarser a noop.