pub struct MutationTask<'a, N: Grammar, H: TaskHandle = TriggerHandle, S: SyncBuildHasher = RandomState> { /* private fields */ }Expand description
A task that grants access to change the content of the documents managed by the Analyzer.
This kind of task implements a MutationAccess trait through which you can create, delete, or edit the existing documents’ content, but you cannot read attributes of the semantic graph.
You may have as many instances of this task as needed at the same time, and you can use them from multiple threads to manage distinct documents.
The analyzer allows you to edit independent documents concurrently, but if two independent threads would edit the same document, one of them will block until another one finishes its job.