pub struct DebugObserver<N: Node> { /* private fields */ }Expand description
An observer that prints parsing steps to stdin.
You can provide this function to the ImmutableSyntaxTree::parse_with_observer to test particular syntax parser steps against an arbitrary token input.
Alternatively, consider using Node::debug where you can use arbitrary text instead of a predefined token stream.